.bk-root {
  @charset "UTF-8";
  /*!
 * Pikaday
 * Copyright © 2014 David Bushell | BSD & MIT license | https://dbushell.com/
 */
  /*
clear child float (pika-lendar), using the famous micro clearfix hack
http://nicolasgallagher.com/micro-clearfix-hack/
*/
  /* styling for abbr */
}
.bk-root .pika-single {
  z-index: 9999;
  display: block;
  position: relative;
  color: #333;
  background: #fff;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}
.bk-root .pika-single:before,
.bk-root .pika-single:after {
  content: " ";
  display: table;
}
.bk-root .pika-single:after {
  clear: both;
}
.bk-root .pika-single.is-hidden {
  display: none;
}
.bk-root .pika-single.is-bound {
  position: absolute;
  box-shadow: 0 5px 15px -5px rgba(0, 0, 0, 0.5);
}
.bk-root .pika-lendar {
  float: left;
  width: 240px;
  margin: 8px;
}
.bk-root .pika-title {
  position: relative;
  text-align: center;
}
.bk-root .pika-label {
  display: inline-block;
  position: relative;
  z-index: 9999;
  overflow: hidden;
  margin: 0;
  padding: 5px 3px;
  font-size: 14px;
  line-height: 20px;
  font-weight: bold;
  background-color: #fff;
}
.bk-root .pika-title select {
  cursor: pointer;
  position: absolute;
  z-index: 9998;
  margin: 0;
  left: 0;
  top: 5px;
  opacity: 0;
}
.bk-root .pika-prev,
.bk-root .pika-next {
  display: block;
  cursor: pointer;
  position: relative;
  outline: none;
  border: 0;
  padding: 0;
  width: 20px;
  height: 30px;
  /* hide text using text-indent trick, using width value (it's enough) */
  text-indent: 20px;
  white-space: nowrap;
  overflow: hidden;
  background-color: transparent;
  background-position: center center;
  background-repeat: no-repeat;
  background-size: 75% 75%;
  opacity: 0.5;
}
.bk-root .pika-prev:hover,
.bk-root .pika-next:hover {
  opacity: 1;
}
.bk-root .pika-prev,
.bk-root .is-rtl .pika-next {
  float: left;
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAeCAYAAAAsEj5rAAAAUklEQVR42u3VMQoAIBADQf8Pgj+OD9hG2CtONJB2ymQkKe0HbwAP0xucDiQWARITIDEBEnMgMQ8S8+AqBIl6kKgHiXqQqAeJepBo/z38J/U0uAHlaBkBl9I4GwAAAABJRU5ErkJggg==');
}
.bk-root .pika-next,
.bk-root .is-rtl .pika-prev {
  float: right;
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAeCAYAAAAsEj5rAAAAU0lEQVR42u3VOwoAMAgE0dwfAnNjU26bYkBCFGwfiL9VVWoO+BJ4Gf3gtsEKKoFBNTCoCAYVwaAiGNQGMUHMkjGbgjk2mIONuXo0nC8XnCf1JXgArVIZAQh5TKYAAAAASUVORK5CYII=');
}
.bk-root .pika-prev.is-disabled,
.bk-root .pika-next.is-disabled {
  cursor: default;
  opacity: 0.2;
}
.bk-root .pika-select {
  display: inline-block;
}
.bk-root .pika-table {
  width: 100%;
  border-collapse: collapse;
  border-spacing: 0;
  border: 0;
}
.bk-root .pika-table th,
.bk-root .pika-table td {
  width: 14.28571429%;
  padding: 0;
}
.bk-root .pika-table th {
  color: #999;
  font-size: 12px;
  line-height: 25px;
  font-weight: bold;
  text-align: center;
}
.bk-root .pika-button {
  cursor: pointer;
  display: block;
  box-sizing: border-box;
  -moz-box-sizing: border-box;
  outline: none;
  border: 0;
  margin: 0;
  width: 100%;
  padding: 5px;
  color: #666;
  font-size: 12px;
  line-height: 15px;
  text-align: right;
  background: #f5f5f5;
}
.bk-root .pika-week {
  font-size: 11px;
  color: #999;
}
.bk-root .is-today .pika-button {
  color: #33aaff;
  font-weight: bold;
}
.bk-root .is-selected .pika-button,
.bk-root .has-event .pika-button {
  color: #fff;
  font-weight: bold;
  background: #33aaff;
  box-shadow: inset 0 1px 3px #178fe5;
  border-radius: 3px;
}
.bk-root .has-event .pika-button {
  background: #005da9;
  box-shadow: inset 0 1px 3px #0076c9;
}
.bk-root .is-disabled .pika-button,
.bk-root .is-inrange .pika-button {
  background: #D5E9F7;
}
.bk-root .is-startrange .pika-button {
  color: #fff;
  background: #6CB31D;
  box-shadow: none;
  border-radius: 3px;
}
.bk-root .is-endrange .pika-button {
  color: #fff;
  background: #33aaff;
  box-shadow: none;
  border-radius: 3px;
}
.bk-root .is-disabled .pika-button {
  pointer-events: none;
  cursor: default;
  color: #999;
  opacity: 0.3;
}
.bk-root .is-outside-current-month .pika-button {
  color: #999;
  opacity: 0.3;
}
.bk-root .is-selection-disabled {
  pointer-events: none;
  cursor: default;
}
.bk-root .pika-button:hover,
.bk-root .pika-row.pick-whole-week:hover .pika-button {
  color: #fff;
  background: #ff8000;
  box-shadow: none;
  border-radius: 3px;
}
.bk-root .pika-table abbr {
  border-bottom: none;
  cursor: help;
}
