"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("../root");
require("./slickgrid");
var _a = require("../../core/dom");
_a.styles.append(".bk-root .bk-data-table {\n  box-sizing: content-box;\n  font-size: 11px;\n}\n.bk-root .bk-data-table input[type=\"checkbox\"] {\n  margin-left: 4px;\n  margin-right: 4px;\n}\n.bk-root .bk-cell-special-defaults {\n  border-right-color: silver;\n  border-right-style: solid;\n  background: #f5f5f5;\n}\n.bk-root .bk-cell-select {\n  border-right-color: silver;\n  border-right-style: solid;\n  background: #f5f5f5;\n}\n.bk-root .bk-cell-index {\n  border-right-color: silver;\n  border-right-style: solid;\n  background: #f5f5f5;\n  text-align: right;\n  color: gray;\n}\n.bk-root .bk-header-index .slick-column-name {\n  float: right;\n}\n.bk-root .slick-row.selected .bk-cell-index {\n  background-color: transparent;\n}\n.bk-root .slick-cell {\n  padding-left: 4px;\n  padding-right: 4px;\n}\n.bk-root .slick-cell.active {\n  border-style: dashed;\n}\n.bk-root .slick-cell.editable {\n  padding-left: 0;\n  padding-right: 0;\n}\n.bk-root .bk-cell-editor input,\n.bk-root .bk-cell-editor select {\n  width: 100%;\n  height: 100%;\n  border: 0;\n  margin: 0;\n  padding: 0;\n  outline: 0;\n  background: transparent;\n  vertical-align: baseline;\n}\n.bk-root .bk-cell-editor input {\n  padding-left: 4px;\n  padding-right: 4px;\n}\n.bk-root .bk-cell-editor-completion {\n  font-size: 11px;\n}\n");
exports.bk_data_table = "bk-data-table";
exports.bk_cell_index = "bk-cell-index";
exports.bk_header_index = "bk-header-index";
exports.bk_cell_editor = "bk-cell-editor";
exports.bk_cell_select = "bk-cell-select";
