import "../root";
import "./nouislider";
import "widgets/sliders.css";
export declare const bk_slider_value = "bk-slider-value";
export declare const bk_slider_title = "bk-slider-title";
export declare const bk_input_group = "bk-input-group";
